@@ -11,7 +11,7 @@ BEGIN {
 sub startup_banner {
   my $self = shift;
   
-  $self->banner_called($self->host, $self->port);
+  $self->banner_called([$self->host, $self->port]);
   return $self->SUPER::startup_banner(@_);
 }
 
